#!/usr/bin/env python

#	
#	Command_App: A sample framework for using the
#		tkCoLa module. The application does
#		nothing but open a GUI that demonstrates
#		some features of the tkCoLa module.
#
#		JEMWorks Software, 2000
#		http://www.jasonya.com/jemworks
#		jemworks@jasonya.com
#
#	tkCoLa: Tkinter Command Language
#		A mini-language which provides a simplified
#		interface to Tk widgets and allows rapid
#		prototyping of command interfaces and other
#		GUIs in Python.	
#	
#	Jason McDermott 8/26/99, tkcola@jasonya.com
#	
#	
#	

import string, os
import Commands
import Tkinter

COMMAND_FILE = "Command_App.com"
WINDOW_TITLE = "tkCoLa Application"

CLEAR = 0
BOX = 1
CIRCLE = 2

class Command_App(Tkinter.Frame):
	def __init__(self, command_file, window_title="tkCoLa Application", 
			*args, **kw):
		self.command_file = command_file
		self.window_title = window_title
		
		# call an initialization routine with
		#	the unaccounted for keywords
		apply(self._init_hook, args, kw)
		
		# initialize the command structure and
		#	show the resulting GUI with the "main" definition
		self.command_structure = self._init_commands(command_file)
		
	def _init_hook(self, **kw):
		pass
		
	def _init_commands(self, file):	
		com = Commands.Command_Struct(file, parent=self)
		Tkinter.Frame.__init__(self, None)
		self.pack(expand=1, fill="both")
		self._set_wintitle(self.window_title)
		
		com.manager_window(frame=self, command_title="main")
		self.winfo_toplevel().lift()
		
		return com
		
	def _set_wintitle(self, title=None, **kw):
		self.winfo_toplevel().title(title)
		
	def close(self, **kw):
		self.quit()
		
class Sample_App(Command_App):
	def make_canvas(self, frame=None, width=500, height=500, 
			bg='lightblue', scrollbars=1, **kw):
			
		self.canvas_master = frame
		if scrollbars:
			self.scroll_x_frame = Tkinter.Frame(frame)
			self.scroll_x_frame.pack(side="bottom", expand=1, fill="x")
			self.scroll_x = Tkinter.Scrollbar(self.scroll_x_frame, orient="horizontal", width=15)
			self.scroll_x.pack(side="right", expand=1, fill="x")
			
			self.scroll_y_frame = Tkinter.Frame(frame)
			self.scroll_y_frame.pack(side="left", expand=1, fill="y")
			self.scroll_y = Tkinter.Scrollbar(self.scroll_y_frame, width=15)
			self.scroll_y.pack(side="top", expand=1, fill="y")
			
			self.canvas_master = Tkinter.Frame(frame)
			self.canvas_master.pack(side="right")
			
			
		self.view_canvas = Tkinter.Canvas(self.canvas_master, bg=bg, width=width, height=height)
		self.view_canvas.pack()
		
		# set scroll commands
		if scrollbars:
			self.scroll_x['command'] = self.view_canvas.xview
			self.scroll_y['command'] = self.view_canvas.yview
			self.view_canvas['xscrollcommand'] = self.scroll_x.set
			self.view_canvas['yscrollcommand'] = self.scroll_y.set
			
	def draw_sumpin(self, what=BOX, color="red", xloc=250, 
			yloc=250, size=20, **kw):
		
		if not what:
			all = self.view_canvas.find_all()
			self.view_canvas.delete(all)
			self.view_canvas.update()
			return
		
		x1 = xloc + size
		y1 = yloc + size
		box = (xloc, yloc, x1, y1)
			
		if what == BOX:
			self.view_canvas.create_rectangle(box, fill=color)
			
		elif what == CIRCLE:
			self.view_canvas.create_oval(box, fill=color)
	
	def load_file(self, file=None, **kw):
		path, file = os.path.split(file)
		print "Path: %s Filename: %s" % (path, file)
				
	def save_file(self, file="No_File_Specified", **kw):
		path, file = os.path.split(file)
		print "Path: %s Filename: %s" % (path, file)
		

if __name__ == '__main__':
	sample_app = Sample_App(COMMAND_FILE)	
	sample_app.mainloop()
