#!/usr/bin/env python
#	
#	Command_Functions: A module containing
#		  common command functions.
#		  Part of the tkCoLa suite.
#		  See documentation for details of use.
#		  Version 1.0, August 16, 2000, 5-ish
#
#		JEMWorks Software, 2000
#		http://www.jasonya.com/jemworks
#		jemworks@jasonya.com
#
#	tkCoLa: Tkinter Command Language
#		A mini-language which provides a simplified
#		interface to Tk widgets and allows rapid
#		prototyping of command interfaces and other
#		GUIs in Python.	
#
import string
import os
import sys
import copy

try:
	import Tkinter
	from Tkinter import *
except ImportError:
	print "Commands Error:"
	print "\ttkCoLa requires Tk/Tcl 8.0."
	print "\tSorry!"
	sys.exit(0)
	
try:
	from Command_Init import *
except ImportError, module:
	print "Commands Error:"
	print "\tCan't find %s module." % module
	sys.exit(0)

from types import *

# Function to match a given dictionary
# providing tk configuration options
# with those appropriate for the given
# widget and return a configuration dict.
# Can also perform the configuration.
 			
def config_match(widget, config_dict, configure=0):
	match_dict = {}
	c_dict = widget.configure()
	c_keys = c_dict.keys()
	for option in config_dict.keys():
		# is the user option appropriate for
		# our little widget?
		if option in c_keys:
			if len(c_dict[option]) > 3:
				# not an alias
				if c_dict[option][3] == c_dict[option][4]:
					# and has the default value add it
					match_dict[option] = config_dict[option]
					if configure:
						widget.configure({option:config_dict[option]})	
	return match_dict
	
def parse_config(configuration_string, config, manager):
	# mini config parser, will not recognize tuples or dictionaries,
	# 	or other types containing commas or colons- yet.
	
	# make things easier from inside tkCoLa
	parent = manager.parent
		
	split = string.split(configuration_string, ',')
	for pair in split:
		try:
			# get first character
			token = string.strip(pair)[0]
		except IndexError:
			continue
			
		if token == '$':
			config_var = string.strip(pair)[1:]
			if global_vars.has_key(config_var):
				config_dict = global_vars[config_var]
				if type(config_dict) == DictionaryType:
					for key in config_dict.keys():
						config[key] = config_dict[key]
				else:
					print 'Command Error, tk_config: %s not a config dictionary' % config_var
			else:
				print 'Command Error, tk_config: %s not found' % config_var
		else:
			two = string.split(string.strip(pair), ':')
			if not two:
				# no options
				return {}
			try:
			    key = two[0]
			    value = two[1]
			except IndexError:
				# no options
				return {}
				
			config[key] = eval(value)
			try:
				config[key] = string.atoi(value)
			except ValueError:
				pass
			
	return config

# <cntrl-alt-space>
# <dbl-button-1>
# <alt-ctrl-dbl-2>

def parse_eventkey(event):
	TWO = 2
	event = string.strip(event)
	if event[0] == "<":
		if event[(len(event)-1)] == ">":
			return event
		else:
			return None
	
	list = string.split(event, "-")
	event = ""
	finisher = FALSE
	flag = FALSE
	button = FALSE
	for token in list:
		if flag:
			event = "%s-" % event
		flag = FALSE
		if token in COLA_KEY_SPECIAL.keys():
			event = "%s%s" % (event, COLA_KEY_SPECIAL[token])
			flag = TRUE
			finisher = TRUE
		elif token in COLA_KEY_ALT:
			event = "%sAlt" % event
			flag = TRUE
		elif token in COLA_KEY_CNTRL:
			event = "%sControl" % event
			flag = TRUE
		elif token in COLA_KEY_SHIFT:
			event = "%sShift" % event
			flag = TRUE
		elif token in COLA_KEY_OPTION:
			event = "%sOption" % event
			flag = TRUE
		elif token in COLA_BUTTON:
			button = "Button"
		elif token in COLA_DBLBUT:
			button = "Double-Button"
		elif (token in ('1','2','3')) and button:
			event = "%s%s-%s" % (event, button, token)
			flag = TRUE
			finisher = TRUE
		else:
			event = "%sKey-%s" % (event, token[0])
			flag = TRUE
			finisher = TRUE
	
	if event and finisher:
		event = "<%s>" % event
		return event
	else:
		return None

# get command line tokens- this is a separate function
# since we may want to expand it in the future
def command_get_tokens(line):
	return string.split(line, '|')
	
def set_callback(callback):
	if callback:
	    call = callback[0]
	    call_args = callback[1]
	    return call, call_args
	else:
	    return None, None
		
def prefix_process(argument, command_prefix='./'):
	if not argument:
		return ''
	
	if argument[0] == '$':
		index = string.find(argument[1:], '$')
	else:
		index = string.find(argument, '$')
	
	if index == -1:
		command = argument
		args = ''
	else:	
		command = argument[0:index]
		args = process_args(argument[(index+1):], command_prefix)
	
	command = process_special(command, command_prefix)
	
	return command, args

# this routine processes an argument string
# for global_vars and other special things
# in the form '..., keyword = $designator, ...' 	
def process_args(arg_string, command_prefix):
	result = ''
	splits = string.split(arg_string, '=')

	for split in splits:
		if string.strip(split)[0] == '$':
			endex = string.find(split, ',')
			if endex == -1:
				# comma not found
				endex = string.find(split, '.')
				if endex == -1:
					endex = len(split)
			prep = split[1:endex]
			prep = process_special(prep, command_prefix)
			split2 = split[endex:]
			split = prep + split2
			
		result = result + split + '='
		
	# strip trailing '=' char
	result = result[0:(len(result)-1)]
	
	return result

def process_special(arg, command_prefix):
	# alias dict definitions
	# note that these will override global_vars
	# definitions if both have the same key	
	
	arg_end = ''
	index = string.find(arg, '.')
	
	# if the command starts with a '.' add
	# the command prefix to the start
	if index == 0:
		arg = command_prefix + arg
	
	# if the command has a '.' in it the
	# first part needs to be handled
	# specially in case its a variable
	elif index > 0:
		arg_end = arg[index:]
		arg = arg[0:index]
			
	# look in the alias dictionary
	if alias_dict.has_key(arg):
		arg = alias_dict[arg]
			
	# global_vars dictionary
	elif global_vars.has_key(arg):
		arg = 'global_vars[\'%s\']' % arg
	
	# special user accessible commands
	if arg[0:8] in ('manager_', 'command_'):
		arg = 'self.manager.' + arg
		
	return arg + arg_end
