#!/usr/bin/env python

#	
#	Commands: A module to allow prototyping of commands.
#		  Part of the tkCoLa suite.
#		  See documentation for details of use.
#		  Version 1.0, August 15, 2000, 5-ish
#
#		JEMWorks Software, 2000
#		http://www.jasonya.com/jemworks
#		jemworks@jasonya.com
#
#	tkCoLa: Tkinter Command Language
#		A mini-language which provides a simplified
#		interface to Tk widgets and allows rapid
#		prototyping of command interfaces and other
#		GUIs in Python.	
#	
#	Jason McDermott 8/26/99, mcdermoj@ohsu.edu
#	
#	
#	

import string
import os
import sys
import copy
import tkFont

try:
	import Tkinter
	from Tkinter import *
except ImportError:
	print "Commands Error:"
	print "\ttkCoLa requires Tk/Tcl 8.0."
	print "\tSorry!"
	sys.exit(0)
	
try:
	import Command_Script
	from Command_Init import *
	from Command_Classes import *
	from Command_Options import *
	from Command_Functions import *
except ImportError, module:
	print "Commands Error:"
	print module
	sys.exit(0)

from types import *

class Command_Struct:
	def __init__(self, file_name=None, parent=None, default_mem=TRUE, 
			echo=TRUE, scripting_enabled=TRUE):
		self.file = file_name
		self.parent = parent
		self.default_mem = default_mem
		self.echo = echo
		
		self.command_prefix = 'self.parent'
		self.tk_config = {}
		
		# Window flags
		self.window_flag = {}
		
		# Window dictionaries
		self.window_frame = {}
		
		# Menu dictionaries
		self.menu_type = {}
		self.command_menubutton = {}
		self.command_menu = {}
		self.cascade_menus = {}
		self.menu_groups = {}
		self.menu_index = {}
		self.current_command = {}
		self.command_frame = {}
		
		# Set up the global variables dictionary
		global_vars['colors'] = colors
		
		if file_name:
			self.parse_record = self.init_command_file(file_name)
			self.command_list = self.parse_record.command_list
			
		if scripting_enabled:
			self.scripter = Command_Script.Command_Scripts_Manager(self)
		else:
			self.scripter = None
	
	def init_command_file(self, file_name):
		cp_rec = self.load_commands(DEFAULT_COMMANDS_FILE)
		return self.load_commands(file_name, cp_rec)
			
	def load_commands(self, file_name, record=None):
		# Search the module paths for the specified
		#	command file_name. Since the first
		#	path listed is the one where the
		#	calling script resides, this should
		#	turn up our command file.
		
		dirs = sys.path
		lines = None
		
		for dir in dirs:
		    path = os.path.join(dir, file_name)
		    if os.path.exists(path) and os.path.isfile(path):
			    try:
				file = open(path, 'r')
				lines = file.readlines()
				file.close()
			    except IOError:
				print 'Command Error, load: Command file %s unreadable' % file_name
				return None
					
		if not lines:
			print 'Command Error, load: Command file %s not found' % file_name
			return None
				    
		return self.parse_commands(lines, record)
	
	def find_by_title(self, title, **kw):
		for command in self.command_list:
			if command.title == title:
				return command
		return None
		    
	def list_commands(self, list_box, display_groups=None):
		# use the display_groups key if you want to 
		# externally handle groups- this is recommended.
		
		present = None
		
		if display_groups == 'None':
			return present
			
		show_groups = display_groups or self.current_groups
		
		# fill the box with the action commands
		for member in self.command_list:
			if (member.list_command(list_box, show_groups)):
				# if there are any commands listed then
				present = 1
				
		return present
		
	##################################################################
	## Methods pertaining to scripting
	########### FUTURE PLANS
	def start_recording(self, **kw):
		self.recording = TRUE
		
	def stop_recording(self, **kw):
		self.recording = FALSE
	
	##################################################################
	## Methods pertaining to parsing command files
	###########
	def parse_regular(self, record=None, tokens=None):
		# pre_process keywords
		keyword = tokens[0]
		if KEY_ALIAS.has_key(keyword):
			# alias the keyword
			keyword = KEY_ALIAS[keyword]
			
		if not KEYWORDS.has_key(keyword):
			# not a keyword or not a regular one
			return (FALSE, record)
			
		# else
		type,call,extra,fields = KEYWORDS[keyword]
		
		index = 1
		f_str = 'type=keyword,'
		if fields:
			for fd in fields:
				# required input fields, marked by a '*'
				if fd[0] == '*':
					fd = fd[1:]
					required = TRUE
				else:
					required = FALSE
				
				# definition constants	in the form of 'key=value'
				split = string.split(fd, '=')
				if len(split) > 1:
					f_str = '%s%s,' % (f_str, fd)
				else:	
				    try:
					test = tokens[index]
					f_str = '%s%s=tokens[%i],' % (f_str, fd, index)
					index = index + 1
					
				    except IndexError:
					if required:
						raise CommandParseError(keyword, fd, 'required.')
						return (FALSE, record)
			
		c_str = '%s(%s record=record)' % (call, f_str)
		
		if type == OPTION:
			if not record.command:
				raise CommandParseError(keyword, 'option out of sequence')
				return (FALSE, record)
				
			# create the object - make the call anyway		
			#try:
			object = eval(c_str)
			#except (NameError, AttributeError):
			#	# we shouldn't be here!
			#	raise CommandParseError(keyword, 'internal command parse error')
			#	return (FALSE, record)
				
			record.option = object
			record.command.option_add(record.option)
			return (TRUE, record)
		
		elif type == COMMAND:
			# create the object - make the call anyway		
			try:
				object = eval(c_str)
			except (NameError, AttributeError):
				# we shouldn't be here!
				raise CommandParseError(keyword, 'internal command parse error')
				return (FALSE, record)
				
			record.command = object
			record.command_list.append(object)
			record.command.set_groups(record.group_range)
			record.option = None
			return (TRUE, record)
			
		elif type == OPTION_ADD:
			if not record.option:
				raise CommandParseError(keyword, 'no option defined')
				return (FALSE, record)
			if extra and record.option.type not in extra:
				raise CommandParseError(keyword, 'innapropriate option type')
				return (FALSE, record)
			
			c_str = 'record.option.%s' % c_str
			#try:
			eval(c_str)
			#except (NameError, AttributeError):
			#	# we shouldn't be here!
			#	raise CommandParseError(keyword, 'internal command parse error')
			#	return (FALSE, record)
				
			return (TRUE, record)
			
		elif type == COMMAND_ADD:
			if not record.command:
				raise CommandParseError(keyword, 'no command defined')
				return (FALSE, record)
			c_str = 'record.command.%s' % c_str
			#try:
			eval(c_str)
			#except (NameError, AttributeError):
			#	# we shouldn't be here!
			#	raise CommandParseError(keyword, 'internal command parse error')
			#	return (FALSE, record)	
			return (TRUE, record)
			
		elif type == GLOBAL:
			c_str = 'self.%s' % c_str
			#try:
			eval(c_str)
			#except (NameError, AttributeError):
				# we shouldn't be here!
			#	raise CommandParseError(keyword, 'internal command parse error')
			#	return (FALSE, record)	
			return (TRUE, record)
			
		elif type == CASCADE:
			# if there's an option on the record- add to the option
			if record.option:		# Note: we could do this recursively?
				c_str = 'record.option.%s' % c_str
				#try:
				eval(c_str)
				#except (NameError, AttributeError):
				#	# we shouldn't be here!
				#	raise CommandParseError(keyword, 'internal command parse error')
				#	return (FALSE, record)
					
				return (TRUE, record)
			# if there's a command on the record- add to that
			elif record.command:
				c_str = 'record.command.%s' % c_str
				#try:
				eval(c_str)
				#except (NameError, AttributeError):
				#	# we shouldn't be here!
				#	raise CommandParseError(keyword, 'internal command parse error')
				#	return (FALSE, record)	
				return (TRUE, record)
			# else we put it in the global domain
			else:
				c_str = 'self.%s' % c_str
				#try:
				eval(c_str)
				#except (NameError, AttributeError):
					# we shouldn't be here!
				#	raise CommandParseError(keyword, 'internal command parse error')
				#	return (FALSE, record)	
				return (TRUE, record)
		
		return (FALSE, record)	
				
	def parse_commands(self, lines, cp_rec=None):
		cp_rec = cp_rec or Command_Parse_Record(mem=self.default_mem, echo=self.echo, manager=self)
		parent = self.parent
		cp_rec.clear_command()
		
		# pre-process for multiple lines, designated
		# by a $ at the start of the line to be appended
		for n in range(0, len(lines)):
			try:
				old_line = string.strip(lines[n])
			except IndexError:
				# end of pre-processed file
				break
			multi_flag = 0
			start_n = n
			for line in lines[(n+1):]:
			    line = string.strip(line)
			    if line and (line[0] == '$'):
				old_line = '%s %s' % (old_line, line[1:])
				n = n + 1
				multi_flag = 1
			    else:
			    	break
			if multi_flag:
				old_line = old_line + '\n'
				# replace old multiple lines with new
				# concatenated multi-line
				del lines[start_n:(n+1)]
				lines.insert(start_n, old_line)
		
		for line in lines:
			token_list = []
			l = string.split(line, '|')
			for token in l:
				token_list.append(string.strip(token))
			
			# blank line? skip to the next
			if not token_list[0]:
				continue
			
			# parse regular keywords
			try:
				flag, cp_rec = self.parse_regular(record=cp_rec, tokens=token_list)
			except CommandParseError, err:
				err.tell()
					
			# if parse_regular returns something it parsed the line
			if flag:
				continue
								
			# comment line denoted with a #
			elif token_list[0][0] == '#':
				pass
			else:
				print 'Unrecognized definition %s in %s' % (self.file, token_list[0])
					
		return cp_rec


	def add_file(self, filename=None, record=None, **kw):
		# import a command file and attach it to the one we're making
		new_rec = self.load_commands(filename, record=record)
		if not (new_rec.command_list or new_rec.component_dict):
			raise CommandParseError('import', filename, 'No commands or components found.')
			return
		
		# if everything's ok then update these variables			 
		record = new_rec
		record.clear_command()
		
		# FIXME: flag this for verbose output
		print 'Loaded Command File %s' % filename
		
	def add_module(self, module=None, record=None, **kw):
		# import a python module for use with the command file described here
		eval_string = 'import %s' % module
		try:
			exec(eval_string)
		except ImportError:
			raise CommandParseError('module', module, 'not found')
			return None
			
		# FIXME: flag for verbose output
		print 'Loaded Python Module %s' % module
		
		global_vars[module] = eval(module)
		
	def add_var(self, var_name=None, value=None, **kw):
		parent = self.parent
		manager = self
		
		# set the global_vars[var_name] entry to value
		if global_vars.has_key(var_name):
			print 'Replacing existing global variable %s' % var_name
			
		try:
			global_vars[var_name] = eval(value)
			
		except (NameError, AttributeError):
			raise CommandParseError('set_var', value, 'is invalid.')
			return None
	
	def add_range(self, range=None, record=None, **kw):
		record.group_range = []
		record.group_range.append(range)
		record.clear_command()
		
	def add_component(self, title=None, record=None, **kw):
		record.clear_command()
		record.command = Command_Record('__component', record=record)
		record.component_dict[title] = record.command
	
	def add_prefix(self, prefix=None, record=None, **kw):
		# allows specification of a 'base' scope
		# for all command designations that start
		# with a '.' that follow the specification
		record.command_prefix = prefix
		
	def add_tk_config(self, tk_config=None, record=None, **kw):
		record.tk_config = parse_config(tk_config, record.tk_config, self)
		
	def set_tk_config(self, tk_config=None, record=None, **kw):
		record.tk_config = parse_config(tk_config, {}, self)
		
	def set_binding(self, **kw):
		# FIXME: right now don't allow universal bindings
		#		future? don't know.
		pass
		
	##################################################################
	## Methods pertaining to creation and maintenance of menus
	###########
	
	def menu_init(self, *args, **kw):
		if not kw.has_key('menu_frame'):
			if not kw.has_key('frame'):
				return
			else:
				menu_frame = kw['frame']
		else:
			menu_frame = kw['menu_frame']
		if not kw.has_key('menu_groups'):
			menu_groups = 'ALL'
		else:
			menu_groups = kw['menu_groups']
		if not kw.has_key('menu_title'):
			menu_title = 'Commands'
		else:
			menu_title = kw['menu_title']
		if not kw.has_key('command_frame'):
			command_frame = None
		else:
			command_frame = kw['command_frame']
		if not kw.has_key('menu_side'):
			side = LEFT
		else:
			side = kw['menu_side']
		if not kw.has_key('menu_select_color'):
			menu_select_color = 'blue'
		else:
			menu_select_color = kw['menu_select_color']
		
		if not kw.has_key('tearoff'):
			tearoff = 0
		else:
			tearoff = kw['tearoff']
	
		if not kw.has_key('menu_type'):
			menu_type = 'normal'
		else:
			menu_type = kw['menu_type']
			if menu_type not in ('normal', 'cascade'):
				menu_type = 'normal'
		
		self.menu_type[menu_title] = menu_type
		self.command_menubutton[menu_title] = Menubutton(menu_frame, text=menu_title)
		self.command_menubutton[menu_title].pack(side=side)
		self.command_menu[menu_title] = m = Menu(self.command_menubutton[menu_title],
				disabledforeground='blue', tearoff=tearoff)
		self.command_menubutton[menu_title]['menu'] = m
		self.menu_index[menu_title] = None
		self.current_command[menu_title] = None
		self.command_frame[menu_title] = command_frame
		
		self.menu_set_groups(menu_groups=menu_groups, menu_title=menu_title)
		
	def menu_destroy(self, *args, **kw):
		if not kw.has_key('menu_title'):
			print 'No title'
			return
		else:
			menu_title = kw['menu_title']
			
		del self.command_menubutton[menu_title]
		del self.command_menu[menu_title]
		del self.menu_index[menu_title]
		
		if self.window_frame.has_key(menu_title):
			self.manager_close_window(menu_title=menu_title, destroy=1)
		
	def menu_update(self, *args, **kw):
		if not kw.has_key('menu_title'):
			print 'No title'
			return
		else:
			menu_title = kw['menu_title']
		
		if not self.command_menu.has_key(menu_title):
			return
		
		# normal command menu
		
		if self.menu_type[menu_title] == 'normal':	
		    m = self.command_menu[menu_title]
		    
		    # erase current command list
		    m.delete(0, END)
		    
		    index = 0
		    for command in self.command_list:
			    # filter for groups
			    if not command.list_command(current_groups=self.menu_groups[menu_title]):
				    continue
			    
			    index = index + 1
				    			    
			    #if command == self.current_command[menu_title]:
			    #    state = DISABLED
			    # else:
			    
			    # this feature wasn't working very well
			    state = NORMAL
			    
			    accel = ""
			    if command.mkey:
			    	    accel = command.nkey
				    self.command_menubutton[menu_title].winfo_toplevel().bind(command.mkey, 
						lambda event=None, self=self, title=command.title:
						self.manager_window(command_title=title))
				    
			    # add a menu item for each with appropriate
			    # callback (ie. one that passes the index #)
			    m.add_command(label=command.title,
				    state = state, accelerator=accel,
				    command =
				    lambda event=None, self=self, command=command, index=index, title=menu_title:
					    self.menu_select_command(command, index, title))
		    if not index:
			    # No commands present so say so
			    m.add_command(label='No Commands', state=DISABLED)
			    
		    m.update()
		
		####
		# cascading command menu type
		# each group heads a different sub-menu
		  
		else:
		    m = self.command_menu[menu_title]
		    
		    cascade = {}
		
		    # erase current menu
		    m.delete(0, END)
		    
		    index = 0
		    for command in self.command_list:
			    # filter for groups
			    if not command.list_command(current_groups=self.menu_groups[menu_title]):
				    continue
			    
			    if cascade.has_key(command.group_list[0]):
			    	cascade[command.group_list[0]].append(command)
			    else:
			    	cascade[command.group_list[0]] = []
				cascade[command.group_list[0]].append(command)
				
			    index = index + 1
				
		    if not index:
			    # No commands present so say so
			    m.add_command(label='No Commands', state=DISABLED)
			    return
				    
		    for key in cascade.keys():
			mbs = Menu(self.command_menubutton[menu_title], tearoff=1)
			index = 0
			for command in cascade[key]:
				index = index + 1
			    
				#if command == self.current_command[menu_title]:
				#	state = DISABLED
				#else:
				# not working well
				state = NORMAL
				
				accel = ""
				if command.mkey:
					accel = command.nkey
					self.command_menubutton[menu_title].winfo_toplevel().bind(command.mkey, 
						    lambda event=None, self=self, title=command.title:
						    self.manager_window(command_title=title))
				    	
				mbs.add_command(label=command.title,
				    state = state, accelerator=accel,
				    command =
				    lambda event=None, self=self, command=command, index=index, title=menu_title:
					    self.menu_select_command(command, index, title))
					    
			m.add_cascade(label=key, menu=mbs)
			
		    m.update()
		
	def menu_set_groups(self, *args, **kw):
		if not kw.has_key('menu_groups'):
			print 'No Menu Groups'
			return
		else:
			menu_groups = kw['menu_groups']
		if not kw.has_key('menu_title'):
			print 'No Menu Title'
			return
		else:
			menu_title = kw['menu_title']
		
		if type(menu_groups) not in (TupleType, ListType):
			menu_groups = (menu_groups,)
			
		self.menu_groups[menu_title] = menu_groups
		self.menu_update(menu_title=menu_title)
			
	def menu_select_command(self, command, index, title, event=None):
		#if self.menu_index[title]:
		#	self.command_menu[title].entryconfigure(self.menu_index[title], state=NORMAL)
		#self.command_menu[title].entryconfigure(index, state=DISABLED)		
		self.menu_index[title] = index
		command_frame = self.command_frame[title]
		
		self.manager_window(command=command, menu_title=title,
				    command_frame=command_frame, window_config='def')
			
	def get_keywords(self, kw):
		values = {}
		if not kw.has_key('state'):
			values['state'] = CANCEL
		else:
			values['state'] = kw['state']
		
		if not kw.has_key('enclose_command'):
			values['enclose_command'] = None
		else:
			values['enclose_command'] = kw['enclose_command']
			
		if not kw.has_key('menu_title'):
			if not kw.has_key('command'):
				if not kw.has_key('this_command'):
					return
				else:
				    try:
					values['menu_title'] = kw['this_command'].menu_title
				    except AttributeError:
				    	return
			else:
				values['menu_title'] = kw['command'].menu_title
		else:
			values['menu_title'] = kw['menu_title']

		if not kw.has_key('destroy'):
			values['destroy'] = FALSE
		else:
			if not kw['destroy']:
				values['destroy'] = FALSE
			else:
				values['destroy'] = kw['destroy']
			if kw['destroy'] in TRUE_LIST:
				values['destroy'] = TRUE
			elif kw['destroy'] in FALSE_LIST:
				values['destroy'] = FALSE
				
		if not kw.has_key('at'):
			values['at'] = None
			values['at_args'] = None
		else:
			values['at'] = kw['at']
			try:
				values['at_args'] = kw['at_args']
			except KeyError:
				return None	
				
		return values

			
	###########################################################################
	###################    tkCoLa accessible commands
	###########################################################################
	#
	#	command_return_object: places variable values from command
	#				into matching attributes of any number
	#				of objects.
	#
	#	command_default_object: places attribute values from objects
	#				into defaults of matching command options
	#				when used as a default_callback.
	#

	## Set the defaults to the values of the keys in object
	## 	Be sure that object has a __dict__ and that at
	##	least _some_ of the default variables correspond
	##	to the key names in object.	
	def command_default_object(self, objects=None, **keywords):
		if type(objects) not in (ListType, TupleType):
			objects = (objects,)
			
		value_dict = keywords['value_dict']
		for object in objects:
		    try:
			for key in object.__dict__.keys():
			    if value_dict.has_key(key):
				    value_dict[key] = object.__dict__[key]
		    except AttributeError:
			break
		return value_dict
	
	## Set values in the object corresponding to the returned
	##	variable names to the values returned. The key_check
	##	option filters the returned keys so that only values
	##	that already exist in object are assigned.	
	def command_return_object(self, objects=None, **values):
		if type(objects) not in (ListType, TupleType):
			objects = (objects,)	# make it a tuple
			
		for object in objects:
		    for key in values.keys():
		    	key = self.process_key(key)
		    	if not object.__dict__.has_key(key[0]):
				# skip if the key doesn't exist already
				continue
			# we're receiving a list so handle it appropriately
			if (type(values[key[0]]) == TupleType) and key[1]:
				n = 0 
				for member in values[key]:
					try:
					    object.__dict__[key[0]][n].__dict__[key[1]] = member
					except AttributeError:
					    pass
					n = n + 1
			
			# we're receiving a dictionary so handle it		
			elif type(values[key[0]]) == DictType:
				for member in values[key[0]].keys():
					if key[1]:
					    try:
					    	object.__dict[key[0]][member].__dict__[key[1]] = values[key[0]][member]
					    except AttributeError:
					    	pass
					else:
					    object.__dict__[key[0]][member] = values[key[0]][member]
			# we're receiving another data type
			else:
			    object.__dict__[key[0]] = values[key[0]]
				
	def process_key(self, key):
		split = string.split(key, '$')
		try:
			split1 = split[1]
		except IndexError:
			return key, None
		
		return split[0], split[1]
		    
	#	
	# Command Structure user accessible methods
	# These methods allow windows containing particular command
	#	definitions, or command sets to be opened and closed
	#
	#
	
	## Open a window containing the command with the command_title.
	##	If a frame is supplied place options in the frame
		
	def manager_window(self, command_title=None, **kw):
		# to make things more easily accessible from inside tkCoLa
		parent = self.parent
		manager = self
		
		if not command_title:
			command_title = ''
			if not kw.has_key('command'):
				return
			else:
				command = kw['command']
				command_title = command.title
		else:
			command = self.find_by_title(command_title)
			if not command:
				return
		
		# test to see if the app wants us to open this window
		if not command.get_command_flag():
			return
			
		# Make a copy of the command
		# FIX:
		# This needs a fix. If a new window using the same
		# 	command is opened it will supercede the
		# 	previously opened command data.
			
		if not kw.has_key('data'):
			data = None
		else:
			data = kw['data']
			
		if kw.has_key('data_process'):
			data_process = kw['data_process']
			if type(data_process) in (MethodType, FunctionType, ClassType):
				keys = get_kw(kw)
				if data:
					ex_str = 'data = data_process(data, %s, parent=self.parent)' % keys
				else:
					ex_str = 'data = data_process(%s, parent=self.parent)' % keys
				if self.echo:
					print ex_str
					
				try:
					exec(ex_str)
				except CommandCallError, err:
					err.tell('manager_window, data_process')
					return
			if not data:
				return
									
		command.data = data
			
		if not kw.has_key('frame'):
			if not kw.has_key('command_frame'):
				frame = None
			else:
				frame = kw['command_frame']
		else:
			frame = kw['frame']
			if frame in ('none', 'None'):
				frame = None
				
		if not kw.has_key('menu_title'):
			# make up a new name for this one
			menu_title = command_title
		else:
			menu_title = kw['menu_title']
		if not kw.has_key('window_config'):
			window_config = {}
		else:
			window_config = kw['window_config']
			if window_config in ('default', 'Default', 'DEFAULT', 'def', 'DEF'):
				window_config = {'borderwidth':7}
		
		# if return_mode is true we treat the window as a
		#	function which doesn't execute a command
		# 	but rather returns to the calling function
		#	the variables given by the command	
		if kw.has_key('return_mode'):
			return_mode = kw['return_mode']
		else:
			return_mode = FALSE
				
		# open a new window?		
		if not self.window_frame.has_key(menu_title):	
			if not frame:
				if return_mode:
					self.window_frame[menu_title] = Toplevel(self.parent)
				else:
					self.window_frame[menu_title] = Tk()
				self.window_frame[menu_title].winfo_toplevel().protocol('WM_DELETE_WINDOW',
						lambda event=None, self=self, title=menu_title:
						self.manager_close_window(menu_title=title))
				self.window_flag[menu_title] = TRUE
			else:
				self.window_frame[menu_title] = frame
				self.window_flag[menu_title] = FALSE
				
			self.current_command[menu_title] = command
			
			# set configuration for this window
			self.window_frame[menu_title].configure(window_config)
			
			# if the draw_options method returns something then there are
			# 	no interactive options associated with this then simply
			#	return (draw_options will execute the command)
			try:			
			    if command.draw_options(self.window_frame[menu_title], command.options):
				    command.menu_title = menu_title		
				    self.current_command[menu_title] = command
				    self.manager_close_window(menu_title=menu_title)
				    # return so we don't re-open the window we just closed
				    return
			except CommandRunError, err:
				err.tell()
		
		# we already have a window for this menu_title	
		else:
			# if requested command is already open then do stuff with the window
			if self.current_command[menu_title] == command:
				if not command.options:
					self.manager_close_window(menu_title=menu_title)
					command.menu_title = menu_title		
					self.current_command[menu_title] = command
					# return so we don't re-open the window we just closed
					return
				else:
					command.reset_defaults()
					
			# if the requested command is not the currently opened command
			#	then if the current_command exists- clear the command window
			#	then draw the command options
			else:
				if (self.current_command[menu_title]):
					self.clear_manager_window(self.current_command[menu_title], menu_title)
				
				# if the draw_options method returns something then there are
				# 	no interactive options associated with this then simply
				#	return (draw_options will execute the command)
				try:
				    if command.draw_options(self.window_frame[menu_title], command.options):
					    command.menu_title = menu_title		
					    self.current_command[menu_title] = command
					    self.manager_close_window(menu_title=menu_title)
					    # return so we don't re-open the window we just closed
					    return
				except CommandRunError, err:
					err.tell()
					
		command.menu_title = menu_title
		self.current_command[menu_title] = command
		
		if self.window_flag[menu_title]:
			self.window_frame[menu_title].winfo_toplevel().deiconify()
			self.window_frame[menu_title].winfo_toplevel().tkraise()
			self.window_frame[menu_title].winfo_toplevel().title(command.title)
		
		# if we're in return_mode then wait until the window has been closed
		# 	to return control to the calling function
		#
		#	FIXME: this does not seem to work- glitch in TK?
		#
		if return_mode:
			command.return_mode = TRUE
			command.return_value = CANCEL
			
			# keep tk happy while in the loop
			self.window_frame[menu_title].grab_set()
			self.window_frame[menu_title].mainloop()
			
			# ok we're free - now return
			if command.return_value == CANCEL:
				self.window_frame[menu_title].destroy()
				return None	# hopefully our caller can handle this
			else:
				self.window_frame[menu_title].destroy()
				return command.return_value
				
	## Close the window associated with this command - if any	
	def manager_close_window(self, state=None, **kw):		
		values = self.get_keywords(kw)
		if not values:
			print 'Missing Values'
			return
		
		menu_title = values['menu_title']
		
		# if we've already destroyed this command or its gone
		#	for some other reason
		if not self.current_command[menu_title]:
			return
		
		# if we're in return_mode then get a dict of values
		if self.current_command[menu_title].return_mode:
			self.current_command[menu_title].return_mode = FALSE
			if state == OK:
				ret = self.current_command[menu_title].return_values()
				self.current_command[menu_title].return_value = ret
			else:
				self.current_command[menu_title].return_value = state
			self.window_frame[menu_title].quit()
					
		if values['destroy']:
			if self.current_command[menu_title]:
				self.clear_manager_window(self.current_command[menu_title], menu_title)
			self.window_frame[menu_title].destroy()
			try:
			    del self.window_frame[menu_title]
			    del self.window_flag[menu_title]
			    del self.current_command[menu_title]
			except (KeyError, TclError):
			    pass
			
		elif self.window_flag[menu_title]:
			if self.current_command[menu_title]:
				self.clear_manager_window(self.current_command[menu_title], menu_title)
			self.window_frame[menu_title].winfo_toplevel().withdraw()
		else:
			if self.current_command[menu_title]:
				self.clear_manager_window(self.current_command[menu_title], menu_title)
		
		# if there's a menu associated with this window	
		if self.command_menu.has_key(menu_title):
			self.menu_update(menu_title=menu_title)	
				
		self.current_command[menu_title] = None
		
	def manager_ok(self, *args, **kw):
		values = self.get_keywords(kw)
		if not values:
			print 'Missing Values'
			return
		
		menu_title = values['menu_title']
		
		# if we're in return_mode then we handle command execution differently
		if not self.current_command[menu_title].return_mode:		
			error = self.current_command[menu_title].execute_command(
					enclose_command=values['enclose_command']) #, kw=kw)
					
			# if there's an error then we need to re-ask the user for input
			if not error:
				self.manager_close_window(menu_title=menu_title, 
						destroy=values['destroy'], state=OK)
			else:
				# display a friendly window
				pass
		
		# we are in a dialog-type mode
		else:
			self.manager_close_window(menu_title=menu_title, 
					destroy=values['destroy'], state=OK)
		
	def manager_apply(self, *args, **kw):
		values = self.get_keywords(kw)
		if not values:
			print 'Missing Values'
			return
		
		menu_title = values['menu_title']
				
		error = self.current_command[menu_title].execute_command(
					enclose_command=values['enclose_command']) #, kw=kw)
		if error:
			# display a friendly window
			pass

	def manager_applythen(self, *args, **kw):
		# making things easier from inside tkCoLa
		parent = self.parent
		manager = self
		
		values = self.get_keywords(kw)
		if not values:
			print 'Missing Values'
			return
		
		try:
		    menu_title = values['menu_title']
		    pass_args = values['at_args']
		    atcommand = values['at']
		except KeyError:
		    print 'Command Manager: Error in Command definition'
		    return
		
		command = self.current_command[menu_title]		
		error = command.execute_command(enclose_command=values['enclose_command']) #, kw=kw)
		
		# if there's an error then we need to re-ask the user for input
		if error:
			# display a friendly window
			pass
			
		
		# execute the given command with its args
		eval(atcommand + '(' + pass_args + ')')
		
	def manager_cancel(self, *args, **kw):
		values = self.get_keywords(kw)
		if not values:
			print 'Missing Values'
			return
			
		menu_title = values['menu_title']
		self.manager_close_window(menu_title=menu_title, destroy=values['destroy'], state=CANCEL)
		
	def clear_manager_window(self, command, menu_title):
		# to make things more easily accessible from inside tkCoLa
		parent = self.parent
		manager = self
		
		if command.original_refresh:
			exec(command.original_refresh + '= command.refresh_callback')
			
		if self.window_frame.has_key(menu_title):
			frame_kids = self.window_frame[menu_title].winfo_children()
			for w in frame_kids:
				w.destroy()
	
	# clean up all the remaining manager menu groups
	#	this should be called when closing the application
	def exit_manager(self, **kw):
		# clean up all active windows
		for key in self.command_menu.keys():
			self.menu_destroy(menu_title=key)
	
		    	
def NullCommand():
	pass
		
class Command_Record:
	def __init__(self, title=None, command=None, scope=None,
			return_mode=FALSE, mkey=None, record=None, **kw):
		self.title = title
		
		call = prefix_process(command, record.command_prefix)
		self.command, self.args = set_callback(call)
		
		self.scope = scope
		self.parent = record.parent
		self.manager = record.manager
		# remember input values as default
		self.mem = record.mem
		# echo commands to the screen
		self.echo = record.echo
		
		self.options = []
		self.group_list = []
		self.post_commands = []
		self.binding_list = []
		
		self.static = ''
		self.label_opt = None
		self.default_callback = None
		self.command_flag = None
		self.cflag_args = None
		self.original_refresh = None
		self.window_frame = None
		self.object = None
		
		# dictionary for quick find by var_name
		#	which should pretty much be unique
		#	for any single command
		self.options_hash = {}
		
		# mkey is the event key binding that
		#	will open this command window.
		#	Binding will only happen if this command
		#	is included in a command menu.
		if mkey:
			self.mkey = parse_eventkey(mkey)
			self.nkey = mkey
		else:
			self.mkey = None
			self.nkey = None
		
		# return_mode should be false normally
		self.return_mode = FALSE
		self.return_value = FALSE
		
		self.default_values = ''
		
		# make a copy otherwise this will effect parental options
		# since we're working with a dictionary
		self.tk_config = copy.copy(record.tk_config)
	
	#######################################
	#
	# tkCoLa Access Methods
	# 	These can be called through keywords directly
	#	from the command language.
	#
	##############	
		
	def add_group(self, group_name, **kw):
		self.group_list.append(group_name)
		
	def default_callback_set(self, default_callback=None, refresh_callback=None, record=None, **kw):
		# make things easier from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		call = prefix_process(default_callback, record.command_prefix)
		self.default_callback, self.default_args = set_callback(call)
		call = prefix_process(refresh_callback, record.command_prefix)
		self.refresh_callback, self.refresh_args = set_callback(call)
		
		if self.refresh_callback:
			# store this in string form too
			self.original_refresh = self.refresh_callback
			
			try:
				self.refresh_callback = eval(self.refresh_callback)
				
			except (NameError, AttributeError):
				raise CommandParseError('default_callback', 'refresh_callback', 'does not exist')
				return None
			
	def add_command_flag(self, flag_object=None, record=None, **kw):
		call = prefix_process(flag_object, record.command_prefix)
		self.command_flag, self.cflag_args = set_callback(call)
		
	def add_post_command(self, command=None, pass_flag=FALSE, record=None, **kw):
		call = prefix_process(command, record.command_prefix)
		call, args = set_callback(call)
		if pass_flag in TRUE_LIST:
			pass_flag = TRUE
		else:
			pass_flag = FALSE
			
		self.post_commands.append((call, args, pass_flag))
	
	def add_var(self, var_name=None, value=None, **kw):	
		parent = self.parent
		manager = self
		
		# set local variable to value
		if self.__dict__.has_key(var_name):
			print 'Replacing existing local variable %s' % var_name
			
		try:
			self.__dict__[var_name] = eval(value)
			
		except (NameError, AttributeError):
			raise CommandParseError('set_var', value, 'is invalid.')
			return None
		
	def label_set(self, label=None, record=None, **kw):
		if not self.label_opt:
			self.label_opt = Label_Option(label=label, record=record)
		else:
			self.label_opt.label = self.label_opt.label + ' ' + label
			
	def static_add(self, var_val, **kw):
		if self.static:
			self.static = self.static + ',' + var_val
		else:
			self.static = var_val
	
	def insert_component(self, component=None, record=None, **kw):
		if not record.component_dict.has_key(component):
			raise CommandParseError('insert_component', component, 'not found')
			
		elif not record.command:
			raise CommandParseError('insert_component', component, 'not inserted, no command')
		else:
			# add component options to end of current command
			# command options- default behavior.
			# command.static_add(component.static)
			
			# add component bindings
			for bind in record.component_dict[component].binding_list:
				record.command.binding_list.append(bind)
				
			for member in record.component_dict[component].options:
				member.command = None
				insert_option = copy.copy(member)
				insert_option.command = record.command
				if not insert_option.config_flag:
					insert_option.tk_config = copy.copy(record.command.tk_config)
				
				record.command.option_add(insert_option)
						
	def add_tk_config(self, tk_config=None, record=None, **kw):
		self.tk_config = parse_config(tk_config, self.tk_config, self.manager)
		
	def set_tk_config(self, tk_config=None, record=None, **kw):
		self.tk_config = parse_config(tk_config, {}, self.manager)
		
	def set_binding(self, event=None, callback=None, top=FALSE, record=None, **kw):
		call = prefix_process(callback, record.command_prefix)
		callback, args = set_callback(call)
		
		event = parse_eventkey(event)
		if event:
			self.binding_list.append((event, callback, args, top))
		
	def configure_bindings(self, **kw):
		if not self.binding_list:
			return
		
		for (event, call, args, top) in self.binding_list:
		    if top:
		    	key = self.options[0].widgets.keys()[0]
			wid = self.options[0].widgets[key]
		    	wid.winfo_toplevel().bind(event, 
				    lambda event=None, self=self, call=call, args=args:
				    self.handle_callback(callback=call, call_args=args, 
				    pass_flag=FALSE))
		    else:
			for option in self.options:
			    for key in option.widgets.keys():
				option.widgets[key].bind(event, 
				    lambda event=None, self=self, call=call, args=args:
				    self.handle_callback(callback=call, call_args=args, 
				    pass_flag=FALSE))

				
	#######################################
	#
	# User Access Methods
	# 	These methods may be useful from
	#	Python or tkCoLa.
	#
	##############	
		
	# callback for options to be able to toggle the activation
	#	state of other options given by the var_name, option
	#	or label attributes of the option
	#
	#	Note: simply toggling the options raises some
	#		problems with options that may change
	#		their states. Linking (below) is a
	#		better alternative.
	#
	def toggle_option(self, var_name=None, option=None, label=None, **kw):
		if not var_name or option or label:
			return None
		
		if var_name:
		    if type(var_name) in (ListType, TupleType):
		    	for name in var_name:
				self.toggle_option(var_name=name)
		    	return
			
		    for opt in self.options:
		    	if opt.__dict__.has_key('var_name'):
			    if var_name == opt.var_name:
				    opt.toggle()
				
		elif option:
		    if type(option) in (ListType, TupleType):
		    	for opt in option:
				self.toggle_option(option=opt)
			return
			
		    for opt in self.options:
			if option == opt:
				opt.toggle()
				
		elif label:
		    if type(label) in (ListType, TupleType):
		    	for lab in label:
				self.toggle_option(label=lab)
			return
			
		    for opt in self.options:
		    	if opt.__dict__.has_key('label'):
			    if label == opt.label:
				    opt.toggle()
	
	#
	# method: link_option(this_opt, var_name, option, label, invert)
	#		Links the state of the options specified by
	#			the var_name(s), option(s), or label(s)
	#			to the state of the calling option. The
	#			calling option should be a checkbutton (for now)
	#			and the specified target options will be
	#			on when the calling option is and off when
	#			its off, or inverted if invert is true.
	#			    
	def link_options(self, this_option=None, var_name=None, option=None, 
					label=None, invert=None, **kw):
		if not var_name or option or label:
			return None
			
		if type(this_option) != InstanceType:
			return
			
		if this_option.type != "check":
			return
			
		if this_option.get()[1] == this_option.on_value:
			if invert:
				check_on = DISABLED
			else:
				check_on = NORMAL
		else:
			if invert:
				check_on = NORMAL
			else:
				check_on = DISABLED
		
		if var_name:
		    if type(var_name) in (ListType, TupleType):
		    	for name in var_name:
				self.link_options(this_option=this_option, var_name=name, invert=invert)
		    	return
			
		    for opt in self.options:
		    	if opt.__dict__.has_key('var_name'):
			    if var_name == opt.var_name:
				    opt.put_state(check_on)
				
		elif option:
		    if type(option) in (ListType, TupleType):
		    	for opt in option:
				self.link_options(this_option=this_option, option=opt, invert=invert)
			return
			
		    for opt in self.options:
			if option == opt:
				opt.put_state(check_on)
				
		elif label:
		    if type(label) in (ListType, TupleType):
		    	for lab in label:
				self.link_options(this_option=this_option, label=lab, invert=invert)
			return
			
		    for opt in self.options:
		    	if opt.__dict__.has_key('label'):
			    if label == opt.label:
				    opt.put_state(check_on)
	
	# Similar to above but simpler since the method only takes
	#	variable names or lists of variable names.
	#	This can also be used to link one set of vars
	#	one way and another another way at the same time.
	def link_vars(self, this_option=None, on_vars=None, off_vars=None, 
			on_var=None, off_var=None, **kw):
				
		on_vars = on_vars or on_var
		off_vars = off_vars or off_var
		
		if not on_vars and not off_vars:
			return None
			
		if type(this_option) != InstanceType:
			return
			
		if this_option.type != "check":
			return
			
		if this_option.get()[1] == this_option.on_value:
			check_on = NORMAL
			check_off = DISABLED
		else:
			check_on = DISABLED
			check_off = NORMAL
		
		if on_vars:
		    if type(on_vars) in (ListType, TupleType):
		    	for name in on_vars:
				self.link_vars(this_option=this_option, on_var=name)
		    else:
		    	if self.options_hash.has_key(on_vars):
				self.options_hash[on_vars].put_state(check_on)
			#for opt in self.options:
			#    if opt.__dict__.has_key('var_name'):
			#	if on_vars == opt.var_name:
			#		opt.put_state(check_on)
					
		if off_vars:
		    if type(off_vars) in (ListType, TupleType):
		    	for name in off_vars:
				self.link_vars(this_option=this_option, off_var=name)
		    else:
		    	if self.options_hash.has_key(off_vars):
				self.options_hash[off_vars].put_state(check_off)
			#for opt in self.options:
			#    if opt.__dict__.has_key('var_name'):
			#	if off_vars == opt.var_name:
			#		opt.put_state(check_off)	    
	
	#
	# method: get_option(var_name)
	#		Given a variable name, label, type,
	#		returns an option. If ltype is specified
	#		this will return a list of options with
	#		the specified type.
	#		Must be valid otherwise
	#		will return None which will screw
	#		things up on the other end.
	#
	
	def get_option(self, var_name=None, label=None, type=None, ltype=None, **kw):
		if ltype:		# returns a list of all options with specified type
			list = []
			for opt in self.options:
				if opt.type == ltype:
					list.append(opt)
			return list
		
		if var_name:
			find = var_name
			key = 'var_name'
		elif label:
			find = label
			key = 'label'
		elif type:
			find = type
			key = 'type'	# will return the first option of specified type	
		else:
			# this will bounce an error
			# for our tkCoLa caller- problem?
			return None
			
		for opt in self.options:
		    if opt.__dict__.has_key(key):
			if opt.__dict__[key] == find:
				return opt
		print 'CommandError: Option not found var_name=%s' % var_name
		return None
	
	# method: get_previous(index)
	#		Returns the option index away from
	#		this_option in the options list.
	#
	def get_previous(self, index=-1, this_option=None, **kw):
		n = 0
		found = FALSE
		for opt in self.options:
			if this_option == opt:
				found = TRUE
				break
			n = n + 1
		
		if not found:
			return None
			
		opt_n = n + index
		if opt_n < 0 or opt_n > len(self.options):
			return None
			
		else:
			return self.options[opt_n]	
				    
	def set_groups(self, groups, **kw):
		self.group_list = groups		

	#######################################
	#
	# Command Core Methods
	# 	These methods are for internal use (mostly).
	#
	##############	
	def option_add(self, option):
		self.options.append(option)
		if option.var_name:
			if self.options_hash.has_key(option.var_name):
				# FIXME: raise an error- but don't stop?
				print "Command Parse Error: Duplicate var_name %s found" % option.var_name
			self.options_hash[option.var_name] = option
	
	def clear_groups(self):
		self.group_list = []
	
	def get_command_flag(self):
		# make things easier from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		# default is to open the command
		if not self.command_flag:
			return 1
		
		try:
		    val = eval(self.command_flag)
		except AttributeError:
			print "Command Error, command_flag: %s not an attribute" % self.command_flag
			return 0
		except IndexError, index:
			print "Command Error, command_flag: %d not an index for %s" % (index, self.command_flag)
			return 0
		except KeyError, key:
			print "Command Error, command_flag: %s not a key for %s" % (key, self.command_flag)
			return 0
		if type(val) in (MethodType, FunctionType):
			com_string = "%s(%s)" % (self.command_flag, self.cflag_args)
			val = eval(com_string)
		
		return val
				
	def refresh_defaults(self, selected=None):
		# this is lame and only allows one value to be
		# sent through and it can't be a keyword
		# I know there's a better way to do this
		# using *args, **keywords
		# but don't want to deal with it now.
		
		# make things easier from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		eval('self.refresh_callback(selected,' + self.refresh_args + ')')
		
		self.reset_defaults()
		
	def reset_defaults(self):
		self.set_defaults(self.options)
		for opt in self.options:
			if opt.type in ('entry', 'check', 'radio', 'scale', 'button', 'pop'):
				opt.update_default()
	
	def set_defaults(self, option_list):
		# to make things more easily accessible from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		value_dict = {}
		
		# generate a list of variables and values
		for opt in option_list:
			if opt.type == 'text' and opt.var_name:
				value_dict[opt.var_name] = opt.default
			if opt.type in ('entry', 'check', 'radio', 'scale', 'button', 'rebutton', 'pop'):
				value_dict[opt.var_name] = opt.default
				
		# default callback is responsible for taking the
		# dictionary and replacing the default values with
		# new application/data specific default values.
		if self.default_args:
			default_args = self.default_args + ','
		else:
			default_args = self.default_args
			
		command_string = ('value_dict = ' + self.default_callback +
				 '(' + default_args + 'value_dict=value_dict)')
		exec(command_string)
		
		# place the new list in the defaults for the options
		for opt in option_list:
			if opt.type in ('entry', 'rebutton', 'frame'):
				try:
					opt.default = str(value_dict[opt.var_name])
				except KeyError:
					# if we've been given an option with
					# no corresponding default then skip
					continue
				
				# override the memorized default value as well
				opt.default_mem = opt.default
				
			elif opt.type == 'scale':
				try:
					opt.default = str(value_dict[opt.var_name])
				except KeyError:
					continue
				opt.default_mem = opt.default
				
			elif opt.type == 'check':
				try:
					if str(value_dict[opt.var_name]) == opt.on_value:
						opt.default = 'on'
						opt.default_mem = 'on'
					else:
						opt.default = 'off'
						opt.default_mem = 'off'
				except KeyError:
					continue
					
			elif opt.type == 'radio':
				try:
					for option in opt.radio_options_list:
						if str(value_dict[opt.var_name]) == option[1]:
						    opt.default = option[0]
						    opt.default_mem = option[0]
				except KeyError:
					continue
					
			elif opt.type == 'pop':
				try:
					for option in opt.item_list:
						if str(value_dict[opt.var_name]) == option[1]:
							opt.default = option[1]
							opt.default_mem = option[1]
							self.variable = option[1]
				except KeyError:
					continue
			
			# special case of insertion		
			elif opt.type == 'text':
				if opt.var_name:
				    try:
					opt.default = value_dict[opt.var_name]
				    except KeyError:
				    	continue
							
	def draw_options(self, frame, option_list, no_label=0, frame_opts=None):
		# to make things more easily accessible from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		# set the defaults if there's a callback
		if self.default_callback:
			self.set_defaults(option_list)
			
			if self.original_refresh:
			    # set this weird redirect, ie. every time the
			    # refresh_callback is called it will first reset
			    # the command option defaults and update them.
			    exec(self.original_refresh + '= self.refresh_defaults')
		
		# if there are no command_options then
		# we immediately execute this command
		if not option_list:
			self.execute_command()
			# close the manager window if applicable
			return 1
		
		config_match(frame, self.tk_config, configure=1)
		
		# make the frames dictionary accessible for 
		# passing frames to callbacks, etc.	
		self.frames = frames = {}
		frames['_top'] = frame
		last_frame = frame
		
		# Show helpful text at the bottom
		if self.label_opt and not no_label:
			label_frame = Frame(frame)
			label_frame.pack(side=BOTTOM)
			self.label_opt.draw_widgets(label_frame)
						
		# display the option widgets
		frame_opts = frame_opts or {'side':TOP, 'padx':4, 'pady':4}
		for opt in option_list:
			if opt.type in ('col', 'row'):
			    if opt.type == 'col':
				frame_pack = {'side':opt.side, 'fill':Y, 'expand':1, 'padx':opt.padx}
				frame_opts = {'side':TOP, 'pady':opt.pady}
				
			    elif opt.type == 'row':
				frame_pack = {'side':opt.side, 'fill':X, 'expand':1, 'pady':opt.pady}
				frame_opts = {'side':LEFT, 'padx':opt.padx}
			    
			    if opt.in_frame and frames.has_key(opt.in_frame):
				target_frame = frames[opt.in_frame]
			    else:
			    	target_frame = last_frame
			    	
			    if opt.label:
			    	frames[opt.label] = frame = opt.draw_option(target_frame)
				last_frame = frame
			    else:
			    	frame = opt.draw_option(target_frame)
			    
			    frame.pack(frame_pack)
			
			opt.draw_option(frame, frame_opts=frame_opts)
			
		frame.update()
		
		self.configure_bindings()
		
	def return_values(self):
		parent = self.parent
		manager = self.manager
		command = self
		
		value_dict = self.get_value_dict()
		if value_dict == VAL_ERROR:
			return VAL_ERROR
		
		# parse up the static variables
		if self.static:
		    split = string.split(self.static, ',')
		    for glob in split:
			    var, val = string.split(glob, '=')
			    value_dict[var] = eval(val)
			
		return value_dict
	
	def get_values(self):
		value_string = ''
		for opt in self.options:
		    varval = opt.get()
		    if not varval:
			continue
		    if varval == VAL_ERROR:
		    	return VAL_ERROR
		    var, val = varval
		    
		    value_string = '%s%s=%s,' % (value_string, var, val)
		return value_string
		
	def get_value_dict(self):
		value_dict = {}
		for opt in self.options:
		    varval = opt.get()	
		    if not (varval):
			continue
		    if varval == VAL_ERROR:
		    	return VAL_ERROR
		    var, value = varval
		    value_dict[var] = value
		return value_dict
	
	# Use the add_command option to specify a method
	# which takes a callback (currently in the form of
	# 'command=command') so that expressions such as
	# for_every_do(command=command, args=args) can be
	# executed.
	
	def execute_command(self, enclose_command=None, kw=None):
		# to make things more easily accessible from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		# error should be passed back from the command function 
		#	if input isn't cool.
		error = None
		
		if not self.command:
			print 'Command Error, execute: No command to exectute'
			return
			
		command_string = self.get_command_string(enclose_command, kw)
		if not command_string:
			return
			
		command_string = 'error = %s' % command_string
		exec(command_string)
		
		for (command, args, pass_flag) in self.post_commands:
			command_string = self.get_command_string(pass_vals=args, 
							command=command, pass_flag=pass_flag)
			if not command_string:
				continue
			eval(command_string)
			
	def get_command_string(self, enclose_command=None, pass_vals=None, command=None, pass_flag=1):
		command = command or self.command
		pass_args = ''
		if pass_vals:
		    try:
			for key in pass_vals.keys():
				if type(pass_vals[key]) in (IntType, FloatType, StringType):
					pass_args = (pass_args + '%s=%s,') % (key, pass_vals[key])
				    
		    except AttributeError:
			pass_args = pass_vals + ','
			
		# pass args to command, default
		if pass_flag:    
		    opt_string = self.get_values()
		    if opt_string == VAL_ERROR:
			    return
		    
		    # send the command and its parent back with this call
		    add_string = 'this_command=self, parent=self.parent'	
		    self.default_values = opt_string
		    if self.args:
			    args = self.args + ','
		    else:
			    args = self.args
		    if self.static:
			    static = self.static + ','
		    else:
			    static = self.static
			    
		# do not pass args to command	    
		else:
			args = static = opt_string = add_string = ''
			if pass_args:
			    # remove the comma at the end of pass_args
			    pass_args = pass_args[0:(len(pass_args)-1)]
		 	
		if not enclose_command:			
		    command_string = command + '(' + args + static + opt_string + pass_args + add_string + ')'
		else:
		    command_string = (enclose_command + '(' + 'command=' + command + ',' + args +
					    static + opt_string + pass_args + add_string + ')')
					    
		if self.echo:
			print command_string
		
		return command_string
	
	# handles a callback generated by an option	
	def handle_callback(self, option=None, callback=None, call_args=None, 
			pass_flag=TRUE, event=None):
		# to make things easier from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
	
		com_static = self.static
		
		if option:
			callback = callback or option.callback
			call_args = call_args or option.call_args
		
		if option and (option.type in ('button','rebutton')):
			static_vars = option.static_vars
			if not static_vars:
				static_vars = ''
			if not option.pass_flag or not pass_flag:
				opt_string = ''
				com_static = ''
			else:
				opt_string = self.get_values()
				if opt_string == VAL_ERROR:
					print 'Command: failed get option values'
					return
		else:
			static_vars = ''
			if option and option.type == 'frame':
				opt_string = ''
			elif not pass_flag:
				opt_string = ''
			else:
				opt_string = self.get_values()
				if opt_string == VAL_ERROR:
					print 'Command: failed get option values'
					return
			    
		# if nothing has changed since last time and we're not a button
		# don't bother calling the callback	
		#if ((opt_string == self.default_values) and not 
		#	(option.type in ('button', 'menu', 'frame', 'select', 'multi_select'))):
		#	return
		
		identity = 'this_command=self, this_option=option'
		if call_args:
			call_args = call_args + ','
		
		if com_static:
			com_static = com_static + ','
		if static_vars:
			static_vars = static_vars + ','
		args = com_static + call_args + static_vars + opt_string
		
		callback_string = (callback + '(' + args + identity + ')')
		
		if self.echo:
			print callback_string

		return eval(callback_string)
		
	
	def list_command(self, list_box=None, current_groups='ALL'):
		present = None
		
		# decide which commands to display according to groups
		# if we haven't been sent a list_box then simply return
		# a boolean to indicate if this command is in the groups list
		if not current_groups:
			return 0
			
		if current_groups in ('ALL', 'all', 'All', 'every', 'Every', 'EVERY'):
			if list_box:
			    list_box.insert(END, self.title)
			    present = 1
			else:
			    return 1
		else:
			for group in current_groups:
				if group in self.group_list:
				    if list_box:
					list_box.insert(END, self.title)
					present = 1
				    else:
				    	return 1
		# command isn't in the group list
		if not list_box:
			return 0
			
		return present
	
	# a routine for use by options to get local variables
	# evaluated at the command level.	
	def eval_var_string(self, var_string, option=None):
		# making things easier from inside tkCoLa
		parent = self.parent
		manager = self.manager
		command = self
		
		var = eval(var_string)
		
		# maybe it's a method or function
		if type(var) in (MethodType, FunctionType):
			var_string = '%s()' % var_string
			var = eval(var_string)
		
		return var
		
	## Updates the default values by calling the default_callback
	##	and updating the option values.		    
	def command_set_defaults(self, *args, **kw):
		if default_callback:
			self.reset_defaults()

###########################################################################################################
#
#	Command Module Utility Functions
#
############################################################################################################


## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
#
#	Command Parser Definitions
#		Utility Functions
#
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##	    

# routine to decompress the **kw dict into
# a string suitable for passing along values
# with actual keywords associated with them
# FIX: only processes ints, floats and strings	
def get_kw(kw):
	if not type(kw) == DictType:
		return ''
	arg = ''
	for key in kw.keys():
		this = kw[key]
		if type(this) in (IntType, FloatType):
			this = str(this)
		elif type(this) == StringType:
			this = '\'%s\'' % this
		else:
			continue
		
		arg = '%s,%s=%s' % (arg, key, this)
	
	# cut off the leading comma	
	return arg[1:]
