#! /usr/bin/env python

#	
#	Stack Manager
#		A Python module which establishes
#		and maintains a stack of user documents
#		etc. To be used by an application.
#	
#		Jason McDermott, JEMWorks Software, 8/8/2000
#				
#

import string
import types
import copy

import Tkinter
from Tkinter import *
	
		
class Stack_Manager:
	def __init__(self, select_callback, parent=None,
				mb_frame=None, list_frame=None, 
				stack_limit=100):
		
		self.select_callback = select_callback		
		self.parent = parent
		self.stack_limit = stack_limit
		
		self.stack_list = []
		self.current_selection = 1
		self.current_list = []
		self.current_members = []
		
		# Make the menu and list widgets if needed	
		self.menu = None
		self.list = None
				
		if mb_frame:
			self.make_menu(mb_frame)
		if list_frame:
			self.make_list(mb_frame)
		
	def make_menu(self, frame=None, tearoffcommand=None, title='Stack', **kw):
		tk_add = {}
		if type(tearoffcommand) in (FunctionType, MethodType):
			tk_add['tearoffcommand'] = tearoffcommand
		tk_add['disabledforeground'] = 'blue'
		tk_add['tearoff'] = 1
			
		self.menu_var = IntVar(frame)
		self.menu = 'true'
		self.mb_frame = frame
		self.stackMB = Menubutton(frame, text=title)
		self.stackMB.pack(side=LEFT)
		self.stackMenu = m = Menu(self.stackMB, tk_add)
		self.stackMB['menu'] = m
		self.update_menu()
		
	def make_stacksubmenu(self, menu=None, list_type=FALSE, **kw):
		ops = (('Remove Current',self.remove_member), 
		       ('Copy Current',self.duplicate_member),
		       ('Clear Stack',self.clear_stack),
		       ('separator', 1),
		       ('Select All', self.select_all))
			
		if not menu:
			return
			
		for rec in ops:
			if rec[0] == 'separator':
				if not list_type:
					break
				else:
					menu.add_separator()
			else:
				menu.add_command(label=rec[0], command=rec[1])
		
		return menu
		
	def update_menu(self):
		if not self.menu:
			# no menu to update
			return
			
		m = self.stackMenu
		
		self.opMenu = Menu(self.stackMB, tearoff=1)
		self.make_stacksubmenu(menu=self.opMenu)
		
		# erase current stack list
		m.delete(0, END)
		m.add_cascade(label='Stack Tools', menu=self.opMenu)
		
		n = 0
		
		for member in self.stack_list:
			n = n + 1
			
			if n == self.current_selection:
				state = DISABLED
			else:
				state = NORMAL
			
			# add a menu item for each with appropriate
			# callback (ie. one that passes the index #)
			m.add_command(label=member._title,
				state = state,
				command =
				lambda event=None, self=self, n=n:
					self.select_member(n=n))
		m.update()
	
	def stack_manager(self, event=None):
		pass
							
	def make_list(self, frame=None, width=35, height=16, mode=BROWSE, **kw):
		self.list = TRUE
		
		self.list_frame = master = frame
		self.stack_scroll = Scrollbar(master, width=10)
		self.stack_scroll.pack(side='left', fill='y')
		
		self.stack_listbox = Listbox(master, width=width, height=height, 
						selectmode=mode, exportselection=0)
		self.stack_listbox.pack(fill='both')
		self.stack_listbox['yscrollcommand'] = self.stack_scroll.set
		self.stack_scroll['command'] = self.stack_listbox.yview
		
		self.stack_listbox.bind('<ButtonRelease-1>', self.select_action)
		# self.stack_listbox.bind('<ButtonRelease-2>', self.select_all)
		self.stack_listbox.bind('<Double-Button-1>', self.select_member)
		
		self.stack_listbox.bind('<Up>', self.select_action)
		self.stack_listbox.bind('<Down>', self.select_action)
		self.stack_listbox.bind('<Return>', self.select_all)
		
		self.update_listbox()
		self.select_listmember()
		
	def close_list(self, **kw):
		self.list = FALSE
		
	def update_listbox(self):
		if not self.list:
			# no listbox to update
			return
			
		f = self.stack_listbox
		
		# erase current list
		f.delete(0, END)
		for member in self.stack_list:
			f.insert(END, member._title)
			
	def update(self):
		self.update_menu()
		self.update_listbox()
					
	def select_action(self, event=None):
		sel = self.stack_listbox.curselection()
		if len(sel) == 0:
			return None
			
		n = string.atoi(sel[0])
		
		self.current_list = []
		self.current_members = []
		for x in sel:
			f = string.atoi(x)
			self.current_list.append(f)
			self.current_members.append(self.stack_list[f])
		
		return n+1
		
	def select_all(self, event=None, **kw):
		self.current_list = []
		for n in range(len(self.stack_list)):
			self.current_list.append(n)
		self.select_listmember()
	
	def select_listmember(self, n=None):
		if not self.list:
			return None
			
		if not n:
			n = self.current_selection - 1
		else:
			n = n - 1
		
		for f in range(0,len(self.stack_list)):	
			self.stack_listbox.select_clear(f)	
		
		if len(self.current_list) > 1:
			for n in self.current_list:
				self.stack_listbox.select_set(n)
				self.stack_listbox.see(n)
		else:		
			self.stack_listbox.select_set(n)
			self.stack_listbox.see(n) 
		
	def select_member(self, event=None, n=None, member=None):
		# if we are given a member then do this
		if member:
			self.select_callback(member.data)
			n = 0
			for this in self.stack_list:
				n = n + 1
				if member == this:
					break
		
		previous = self.current_selection
							
		if not n:
			n = self.select_action()
			if not n:
				return
		
		if self.menu:
			self.stackMenu.entryconfigure((previous + 1), state=NORMAL)
			self.stackMenu.entryconfigure((n+1), state=DISABLED)
		
		try:
			if self.list:
			    self.select_listmember(n)
		except TclError:
			self.list = FALSE
		
		self.current_selection = n
		self.current_record = self.stack_list[(n-1)]
		
		# n comes through as n+1
		n = n - 1
		
		# use the callback for selection
		# pass the attached data structure
		# to the callback
		self.select_callback(self.stack_list[n].data)

	def add_member(self, data, title):
		# check for duplicate names
		max = 0
		
		tite = title
		# if the title is already numbered
		# remove the number - note the kludge
		# that only checks the first digit
		if tite[(len(tite)-1)][0] in string.digits:
			title = tite[0]
			for str in tite[1:(len(tite)-1)]:
				title = "%s %s" % (title, str)
		for member in self.stack_list:
			length = len(member._title)-2
			base_title = member._title[0:length]
			if title == base_title:
				num = string.atoi(member._title[length:])
				if num > max:
					max = num
		
		# give the new member a unique name (up to 99)
		# note that every member will have a numeric suffix		
		number = ('%2d' % (max + 1))		
		name = title + number
		
		member = Stack_Record(data, name)
		self.stack_list.append(member)
		if len(self.stack_list) > self.stack_limit:
			del self.stack_list[0]
		
		self.current_record = member
		self.current_selection = len(self.stack_list)
		self.select_member(member=member)	
		self.update()
	
	def set_current_title(self, title):
		self.current_record._title = title
		self.update()
				
	def get_length(self):
		return len(self.stack_list)
	
	def remove_member(self, member=None, *args, **kw):
		# removes currently displayed member
		# FIXME: should have a warning to save along
		# with this command
		
		if len(self.stack_list) > 1:
		    gone_member = member or self.current_record
		    n = -1
		    for entry in self.stack_list:
			    n = n + 1
			    if gone_member == entry:
				    del self.stack_list[n]
					
		    if self.stack_list:
			    if n >= len(self.stack_list):
				    n = n - 1
			    self.current_record = self.stack_list[n]
			    # send n through as n+1 to select_member
			    self.select_member(n=(n+1))
		    else:
			    self.current_record = None
			    
		    self.update()
			
	def duplicate_member(self, member=None, *args, **kw):
		# makes a deepcopy of the member
		# or the current member and adds
		# it to the stack list.
		
		copyfrom = member or self.current_record
		if not copyfrom:
			return
		
		copyto = copy.deepcopy(copyfrom.data)
		self.add_member(copyto, copyfrom._title)
		
	def clear_stack(self, *args, **kw):
		for member in self.stack_list:
			self.remove_member(member=member)
		
	def clone_window(self, *args, **kw):
		# will use a callback to the parent
		# program to allow duplication of the
		# window and the stack 
		pass
		
		
class Stack_Record:
	def __init__(self, data, title, callback=None):
		self._title = title
		self.data = data
		
		# call back for individual records
		# not implemented yet
		self.callback = callback
