#!/usr/local/bin/python

import Tkinter
Tki = Tkinter

class StatusBar(Tki.Frame):
	def __init__(self, master, label, total=None, width=None, color='#99f'):
		self.total = total or 100
		self.label = label
		self.width = width or (master.winfo_reqwidth() / 4)
		self.height = master.winfo_reqheight() / 4
		Tki.Frame.__init__(self, master, relief='raised', bd=2,
				highlightthickness=1, highlightbackground='black',
				width=self.width, height=self.height)
		self.canvas = Tki.Canvas(self, highlightthickness=0,
					width=self.width, height=self.height)
		self.canvas.pack()
		
		self.canvas.bind('<Enter>', self.stop_display)
		self.canvas.bind('<Leave>', self.norm_display)
		self.canvas.bind('<Button-1>', self.stop_command)

		self.labelBorder = 3
		self.bar = self.canvas.create_rectangle(0,0,0,self.height,
					fill=color, outline='',
					width=self.labelBorder)
		self.text = self.canvas.create_text(self.width/2, self.height/2,
					anchor='center',
					text=self.label + ' 0%')
		
		# adjust height to fit text
		boundbox = self.canvas.bbox(self.text)
		self.height = 3 * (boundbox[3]-boundbox[1]) / 2
		
		self.config(width=self.width, height=self.height)
		self.canvas.config(width=self.width, height=self.height)
		self.canvas.coords(self.text, self.width/2, self.height/2)
		
		self.visible = 0
		self.stop_disp = 0
		self.stop = 0
		
	def show(self, makeVisible=1, event=None):
		if makeVisible and not self.stop:
			self.place(relx=0.5,rely=0.4,x=-self.width/2,
				   y=-self.height/2)
		else:
			Tki.Place.forget(self)
		self.visible = makeVisible
		
	def end(self, **kw):
		self.stop = 1
		self.show(0)
		
	def update(self, status):
		canvas = self.canvas
		self.status = status
		barWidth = (self.width * status) / self.total
		canvas.coords(self.bar, -self.labelBorder, -self.labelBorder,
				barWidth + self.labelBorder, 
				self.height + self.labelBorder)
		if self.stop_disp:
			self.stop_display()
		else:
			self.norm_display()
		
	def stop_command(self, event=None):
		self.stop = 1		

	def stop_display(self, event=None):
		self.stop_disp = 1
		self.canvas.itemconfig(self.text, text="Stop %s" % self.label)
		self.canvas.update()
	
	def norm_display(self, event=None):
		self.stop_disp = 0
		self.canvas.itemconfig(self.text, text="%s %d%%" % 
				(self.label,((self.status * 100)/ self.total)))
		self.canvas.update()
		
				
class StatusBar2(Tki.Frame):
	def __init__(self, master, label, total1=None, total2=None, width=None):
		self.total1 = total1 or 100
		self.total2 = total2 or 100
		self.label = label
		self.width = width or (master.winfo_reqwidth() / 4)
		self.height = master.winfo_reqheight() / 4
		Tki.Frame.__init__(self, master, relief='raised', bd=2,
				highlightthickness=1, highlightbackground='black',
				width=self.width, height=self.height)
		
		self.stop = 0		
		#self.stopbutton = Tki.Button(self, text='Stop '+self.label,
		#			command=self.stop_command)
		#self.stopbutton.pack(side='right')
		
		self.canvas = Tki.Canvas(self, highlightthickness=0,
					width=self.width, height=self.height)
		self.canvas.pack()
		
		self.canvas.bind('<Enter>', self.stop_display)
		self.canvas.bind('<Leave>', self.norm_display)
		self.canvas.bind('<Button-1>', self.stop_command)
		
		self.labelBorder1 = 3
		self.labelBorder2 = 10
		self.bar2 = self.canvas.create_rectangle(0,0,0,self.height,
					fill='#f99', outline='',
					width=self.labelBorder2)
		self.bar1 = self.canvas.create_rectangle(0,0,0,self.height,
					fill='#99f', outline='',
					width=self.labelBorder1)
		self.text = self.canvas.create_text(self.width/2, self.height/2,
					anchor='center',
					text=self.label + ' 0%')
		
		# adjust height to fit text
		boundbox = self.canvas.bbox(self.text)
		self.height = 3 * (boundbox[3]-boundbox[1]) / 2
		
		self.config(width=self.width, height=self.height)
		self.canvas.config(width=self.width, height=self.height)
		self.canvas.coords(self.text, self.width/2, self.height/2)
		
		self.visible = 0
		self.stop_disp = 0
		self.status1 = 0
		
	def show(self, makeVisible=1, event=None):
		if makeVisible:
			self.place(relx=0.5,rely=0.4,x=-self.width/2,
				   y=-self.height/2)
		else:
			Tki.Place.forget(self)
		self.visible = makeVisible
		
	def update(self, status1, status2):
		canvas = self.canvas
		self.status1 = status1
		self.status2 = status2
		barWidth1 = self.width * status1 / self.total1
		barWidth2 = self.width * status2 / self.total2
		canvas.coords(self.bar1, -self.labelBorder1, -self.labelBorder1,
				barWidth1 + self.labelBorder1, 
				self.height + self.labelBorder1)
		canvas.coords(self.bar2, -self.labelBorder2, -self.labelBorder2,
				barWidth2 + self.labelBorder2, 
				self.height + self.labelBorder2)
		
		if self.stop_disp:
			self.stop_display()
		else:
			self.norm_display()
	
	def stop_command(self, event=None):
		self.stop = 1		

	def stop_display(self, event=None):
		self.stop_disp = 1
		self.canvas.itemconfig(self.text, text="Stop %s" % self.label)
		self.canvas.update()
	
	def norm_display(self, event=None):
		self.stop_disp = 0
		self.canvas.itemconfig(self.text, text="%s %d%%" % 
				(self.label,((self.status1 * 100)/ self.total1)))
		self.canvas.update()
